use actix_identity::Identity;
use actix_web::{get, http::header::LOCATION, web, HttpResponse, Responder};
use askama_actix::TemplateToResponse;
use serde::Deserialize;
use sqlx::PgPool;

use crate::models::Registration;

use super::ResetPasswordTemplate;

#[derive(Deserialize)]
struct TokenQuery {
    token: String,
}

#[get("/register")]
pub async fn get(
    user: Option<Identity>,
    pool: web::Data<PgPool>,
    query: web::Query<TokenQuery>,
) -> impl Responder {
    if user.is_some() {
        return HttpResponse::Found()
            .insert_header((LOCATION, "/"))
            .finish();
    }

    if let Ok(_) = Registration::does_token_exist(pool.get_ref(), &query.token).await {
        let template = ResetPasswordTemplate {
            token: &query.token,
            title: "Brass - Registrierung",
            endpoint: "/register",
            new_password_label: "Passwort:",
            retype_label: "Passwort wiederholen:",
            submit_button_label: "Registrieren",
        };

        return template.to_response();
    }

    HttpResponse::NotFound().finish()
}
