use actix_web::{http::StatusCode, HttpResponse};
use thiserror::Error;

#[derive(Debug, Error)]
pub enum ApplicationError {
    #[error("unsupported value '{value}' for enum '{enum_name}'")]
    UnsupportedEnumValue { value: String, enum_name: String },
    #[error("unauthorized")]
    Unauthorized,
    #[error("database error")]
    Database(#[from] sqlx::Error),
}

impl actix_web::error::ResponseError for ApplicationError {
    fn status_code(&self) -> StatusCode {
        match *self {
            ApplicationError::UnsupportedEnumValue { .. } => StatusCode::BAD_REQUEST,
            ApplicationError::Unauthorized { .. } => StatusCode::UNAUTHORIZED,
            ApplicationError::Database(_) => StatusCode::INTERNAL_SERVER_ERROR,
        }
    }

    fn error_response(&self) -> HttpResponse {
        HttpResponse::build(self.status_code()).body(self.to_string())
    }
}
